@@ -11,7 +11,7 @@
 #'  
 #'  @examples
 #'  # Create dummy data
-#'  A <- B <- C <- 1:20
+#'  A <- B <- C <- sample(1:20, size = 20, replace = TRUE)
 #'  ID <- sort(rep(seq(1:4), 5))
 #'  Data <- data.frame(ID, A, B, C)
 #'  
@@ -27,23 +27,26 @@
 #' @description The function slides a column up or down to create lag or lead variables. If \code{GroupVar} is specified it will slide \code{Var} for each group. This is important for time-series cross-section data. The slid data is placed in a new variable in the original data frame. 
 #' Note: your data needs to be sorted by date. The date should be ascending (i.e. increasing as it moves down the rows). Also, the time difference between rows should be constant, e.g. days, months, years.
 #' 
-#' @seealso \code{\link{shift}}, \code{\link{ddply}}
+#' @seealso \code{\link{shift}}, \code{\link{dplyr}}
 #'
 #' @source Partially based on TszKin Julian's \code{shift} function: http://ctszkin.com/2012/03/11/generating-a-laglead-variables/
 #'
 #'
-#' @importFrom plyr ddply rename summarize
+#' @importFrom dplyr summarize mutate
 #' @export
 
-slide <- function(data, Var, GroupVar = NULL, NewVar = NULL, slideBy = -1, reminder = TRUE)
+slide <- function(data, Var, GroupVar = NULL, NewVar = NULL, slideBy = -1, 
+                  reminder = TRUE)
 {  
   fake <- total <- NULL
   if (isTRUE(reminder)){
     if (is.null(GroupVar)){
-      message(paste('\nRemember to put', deparse(substitute(data)), 'in time order before running.\n'))      
+      message(paste('\nRemember to put', deparse(substitute(data)), 
+                    'in time order before running.\n'))    
     }
     if (!is.null(GroupVar)){
-      message(paste('\nRemember to order', deparse(substitute(data)), 'by', GroupVar, 'and the time variable before running.\n'))
+      message(paste('\nRemember to order', deparse(substitute(data)), 
+                    'by', GroupVar, 'and the time variable before running.\n'))
     }
   }  
   
@@ -62,14 +65,16 @@ slide <- function(data, Var, GroupVar = NULL, NewVar = NULL, slideBy = -1, remin
 
   # Drop if there are not enough observations per group to slide
   if (!is.null(GroupVar)){
+    data <- eval(parse(text = paste0('group_by(data, ', GroupVar, ')')))  
     data$fake <- 1
-    Summed <- ddply(data, GroupVar, summarize, total = sum(fake))
-    SubSummed <- subset(Summed, total <= abs(slideBy))
+    Minimum <- abs(slideBy) - 1
+    Summed <- dplyr::mutate(data, total = sum(fake))
+    SubSummed <- subset(Summed, total <= Minimum)
     if (nrow(SubSummed) > 0){
-        Dropping <- SubSummed[, GroupVar]
+        Dropping <- unique(SubSummed[, GroupVar])
         data <- data[!(data[, GroupVar] %in% Dropping), ]
-
-        message(paste0('Warning: the following groups have ', abs(slideBy), ' or fewer observations.\nNo reasonable lag/lead can be created, so they are dropped:\n'))
+        message(paste0('\nWarning: the following groups have ', Minimum, 
+                      ' or fewer observations.\nNo reasonable lag/lead can be created, so they are dropped:\n'))
         message(paste(Dropping, collapse = "\n"))
     }
     data <- VarDrop(data, 'fake')
@@ -77,13 +82,16 @@ slide <- function(data, Var, GroupVar = NULL, NewVar = NULL, slideBy = -1, remin
   
   # Create lags/leads
   if (is.null(GroupVar)){
-    data[, NewVar] <- shift(VarVect = VarVect, shiftBy = slideBy, reminder = FALSE)
+    data[, NewVar] <- shift(VarVect = VarVect, shiftBy = slideBy, 
+                            reminder = FALSE)
   }
   else if (!is.null(GroupVar)){
-    vars <- eval(parse(text = paste0("ddply(data[, c(GroupVar, Var)], GroupVar, transform, NewVarX = shift(", Var, ",", slideBy, ", reminder = FALSE))")))
+    DataSub <- eval(parse(text = paste0('group_by(data[, c(GroupVar, Var)], ', 
+                                        GroupVar,')')))
+    vars <- eval(parse(text = paste0("dplyr::mutate(DataSub, NewVarX = shift(", 
+                                 Var, ",", slideBy, ", reminder = FALSE))")))     
     data[, NewVar] <- vars$NewVarX
   }
-  
   return(data)
 }
 
@@ -111,7 +119,8 @@ shift <- function(VarVect, shiftBy, reminder = TRUE){
     stop(paste(shiftBy, 'must be numeric.'), call. = FALSE)
   }
   if (isTRUE(reminder)){
-      message(paste('Remember to put', deparse(substitute(data)), 'in time order before running shift.'))      
+      message(paste('Remember to put', deparse(substitute(data)), 
+                    'in time order before running shift.'))      
   }
 
 
@@ -128,7 +137,7 @@ shift <- function(VarVect, shiftBy, reminder = TRUE){
   } else {
     out <- VarVect
   }
-  out
+  return(out)
 }
 
 #' Create a moving average for a period before or after each time point for a given variable
@@ -145,7 +154,7 @@ shift <- function(VarVect, shiftBy, reminder = TRUE){
 #' 
 #' @examples
 #'  # Create dummy data
-#'  A <- B <- C <- 1:20
+#'  A <- B <- C <- sample(1:20, size = 20, replace = TRUE)
 #'  ID <- sort(rep(seq(1:4), 5))
 #'  Data <- data.frame(ID, A, B, C)
 #'  
@@ -157,26 +166,29 @@ shift <- function(VarVect, shiftBy, reminder = TRUE){
 #'  DataSlidMA2 <- slideMA(data = Data, Var = "B", GroupVar = "ID",
 #'                 NewVar = "BLag_MA", periodBound = -3, offset = 2)
 #' 
-#' @seealso \code{\link{shift}}, \code{\link{slide}}, \code{\link{ddply}}
-#' @importFrom plyr ddply
+#' @seealso \code{\link{shift}}, \code{\link{slide}}, \code{\link{dplyr}}
+#' @importFrom dplyr group_by mutate
 #' @importFrom forecast ma
 #' @export
 
-slideMA <- function(data, Var, GroupVar = NULL, periodBound = -3, offset = 1, NewVar = NULL, reminder = TRUE){
+slideMA <- function(data, Var, GroupVar = NULL, periodBound = -3, offset = 1, 
+                    NewVar = NULL, reminder = TRUE){
   slideBy <- NULL
   if (isTRUE(reminder)){
     if (is.null(GroupVar)){
-      message(paste('Remember to put', deparse(substitute(data)), 'in time order before running slide.'))      
+      message(paste('\nRemember to put', deparse(substitute(data)), 
+                    'in time order before running slide.\n'))      
     }
     if (!is.null(GroupVar)){
-      message(paste('Remember to order', deparse(substitute(data)), 'by', GroupVar, 'and the time variable before running slide.'))
+      message(paste('\nRemember to order', deparse(substitute(data)), 'by', 
+                    GroupVar, 'and the time variable before running slide.\n'))
     }
   }  
   # Determine if Var exists in data
   DataNames <- names(data)
   TestExist <- Var %in% DataNames
   if (!isTRUE(TestExist)){
-    stop(paste(Var, "was not found in the data frame."), call. = FALSE)
+    stop(paste(Var, "was not found in the data frame.\n"), call. = FALSE)
   }
   
   VarVect <- data[, Var]
@@ -188,35 +200,36 @@ slideMA <- function(data, Var, GroupVar = NULL, periodBound = -3, offset = 1, Ne
   Abs = abs(periodBound)
   if (periodBound < 0){
     slideBy = periodBound + abs(offset)
-  }
-  else if (periodBound > 0){
+  } else if (periodBound > 0){
     slideBy = periodBound - abs(offset)
   }
   
-  shiftMA <- function(x, shiftBy = slideBy, Abs = Abs, reminder = reminder){
-    x <- shift(x, shiftBy, reminder = reminder)
-    ma(x, Abs, centre = FALSE)
-  }
-  
   # Create lags/leads moving averages
   if (is.null(GroupVar)){
-    data[, NewVar] <- shiftMA(data[, Var], shiftBy = slideBy, Abs = Abs, reminder = FALSE)
-  }
-  else if (!is.null(GroupVar)){
-    vars <- eval(parse(text = paste0("ddply(data[, c(GroupVar, Var)], GroupVar, transform, NewVarX = DataCombine:::shiftMA(", Var, ", shiftBy =", slideBy, ", Abs = ", Abs, ", reminder = FALSE))"))) 
+    data[, NewVar] <- shiftMA(data[, Var], shiftBy = slideBy, 
+                              Abs = Abs, reminder = FALSE)
+  } else if (!is.null(GroupVar)){
+    DataSub <- eval(parse(text = paste0('group_by(data[, c(GroupVar, Var)], ', 
+                                        GroupVar,')')))
+    vars <- eval(parse(text = paste0("dplyr::mutate(DataSub, NewVarX = shiftMA(", 
+                                    Var, ", shiftBy =", slideBy, ", Abs = ", 
+                                    Abs, ", reminder = FALSE))"))) 
     data[, NewVar] <- vars$NewVarX
+    data <- data.frame(data)
   }
   return(data)
 }
 
 #' Internal function for slideMA
+#' @param x vector
+#' @param shiftBy numeric
+#' @param Abs numeric
+#' @param reminder logical
 #'
-#' @noRd
 #' @keywords internals
-#' @noRd
+#' @export
 
-shiftMA <- function(x, shiftBy = slideBy, Abs = Abs, reminder = reminder){
-  slideBy <- NULL
+shiftMA <- function(x, shiftBy, Abs, reminder){
   x <- shift(x, shiftBy, reminder = reminder)
   ma(x, Abs, centre = FALSE)
 }
@@ -242,15 +255,13 @@ shiftMA <- function(x, shiftBy = slideBy, Abs = Abs, reminder = reminder){
 #'                            spreadBy = 2, reminder = FALSE)
 #'                            
 #' DataSpread2 <- SpreadDummy(data = Data, Var = 'Dummy', GroupVar = 'ID',
-#'                            spreadBy = -2)
-#'                  
+#'                            spreadBy = -2)                 
 #' 
 #' @seealso \code{\link{slide}}
-#' 
-#' @importFrom plyr ddply summarize
 #' @export
 
-SpreadDummy <- function(data, Var, GroupVar = NULL, NewVar = NULL, spreadBy = -2, reminder = TRUE){
+SpreadDummy <- function(data, Var, GroupVar = NULL, NewVar = NULL, 
+                        spreadBy = -2, reminder = TRUE){
     # Check if variable is numeric dummy
     if (class(data[, Var]) != 'numeric'){
         stop(paste(Var, 'must be a numeric dummy variable.'), call. = FALSE)
@@ -323,7 +334,7 @@ SpreadDummy <- function(data, Var, GroupVar = NULL, NewVar = NULL, spreadBy = -2
 #' # Create fake data
 #' ID <- sort(rep(seq(1:4), 5))
 #' Time <- rep(1:5, 4)
-#' Dummy <-  sample(c(0, 1), size = 20, replace = TRUE)
+#' Dummy <-  c(1, sample(c(0, 1), size = 19, replace = TRUE))
 #' Data <- data.frame(ID, Time, Dummy)
 #' 
 #' # Find start/end of spells denoted by Dummy = 1
@@ -344,6 +355,7 @@ StartEnd <- function(data, SpellVar, GroupVar, SpellValue = NULL){
   # Find Start
   Temp <- slide(data = data, Var = SpellVar, GroupVar = GroupVar,
                 slideBy = -1, NewVar = 'TempStart')
+  Temp <- data.frame(Temp)
   Temp$Spell_Start <- 0
   Temp$Spell_Start[is.na(Temp[, SpellVar])] <- NA
   if (is.null(SpellValue)){
@@ -353,9 +365,11 @@ StartEnd <- function(data, SpellVar, GroupVar, SpellValue = NULL){
     Temp$Spell_Start[Temp[, SpellVar] == SpellValue & 
           Temp[, 'TempStart'] != SpellValue] <- 1
   }
+  
   # Find End
   Temp <- slide(data = Temp, Var = SpellVar, GroupVar = GroupVar,
                 slideBy = 1, NewVar = 'TempEnd', reminder = FALSE)
+  Temp <- data.frame(Temp)
   Temp$Spell_End <- 0
   Temp$Spell_End[is.na(Temp[, SpellVar])] <- NA
   if (is.null(SpellValue)){
